# coding=utf-8
import pymssql
import os
import re
import time
import logging
import asyncio
import multiprocessing, threading


class SQLServer:
    errMsg = []
    # filename 指定日志存放文件，level 指定logging级别
    logging.basicConfig(filename=os.path.split(os.path.realpath(__file__))[0] +
                        "/sqlErrorInfo.log",
                        level=logging.INFO)

    def __init__(self, server, port, user, password, database):
        # 类的构造函数，初始化DBC连接信息
        self.server = server
        self.port = port
        self.user = user
        self.password = password
        self.database = database

    def __GetConnect(self):
        # 得到数据库连接信息，返回conn.cursor()
        if not self.database:
            raise (NameError, "没有设置数据库信息")
        self.conn = pymssql.connect(server=self.server,
                                    port=self.port,
                                    user=self.user,
                                    password=self.password,
                                    database=self.database)
        cur = self.conn.cursor()
        if not cur:
            raise (NameError, "连接数据库失败")  # 将DBC信息赋值给cur
        else:
            return cur

    def ExecQuery(self, sql):
        '''
        执行查询语句
        返回一个包含tuple的list，list是元素的记录行，tuple记录每行的字段数值
        '''
        cur = self.__GetConnect()
        cur.execute(sql)  # 执行查询语句
        result = cur.fetchall()  # fetchall()获取查询结果
        # 查询完毕关闭数据库连接
        self.conn.close()
        return result

    def Exec(self, sql, filename):
        try:
            cur = self.__GetConnect()
            cur.execute(
                sql)  # I have experienced problems when executing utf-8
            self.conn.close()
        except Exception as e:
            errorMsg = '[当前时间：%s]SQL文件地址：%s \nSQL语句:%s  \n错误信息：%s \n' % (
                time.strftime("%Y-%m-%d %H:%M:%S",
                              time.localtime()), filename, sql, e)
            self.errMsg.append(errorMsg)
            logging.info(errorMsg)
            logging.info(
                "------------------------------------------------------------------\n"
            )


    # 获取SQL文件中的SQL语句
    def GetSqlInfo(self, filename):
        with open(filename, 'r', encoding='gbk', errors='ignore') as f:
            script = f.read()  # or whatever its encoding is
            script = re.sub(r'\/\*.*?\*\/', '', script,
                            flags=re.DOTALL)  # remove multiline comment
            script = re.sub(r'--.*$', '', script,
                            flags=re.MULTILINE)  # remove single line comment

            sql = []
            do_execute = False
            for line in script.split(u'\n'):
                line = line.strip()
                if not line:
                    continue
                elif line.upper() == u'GO':
                    do_execute = True
                else:
                    sql.append(line)
                    do_execute = line.endswith(u';')

                if do_execute and filter(None,
                                         sql):  # ignore if only blank lines
                    sqlInfo = u'\n'.join(sql)  #.encode("cp1252")
                    # sql.append(sqlInfo)
                    self.Exec(
                        sqlInfo, filename
                    )  # I have experienced problems when executing utf-8
                    do_execute = False
                    sql = []

def main():
    # "开发环境":
    # self.msg = SQLServer(server="127.0.0.1",
    #                      port=0,
    #                      user="test",
    #                      password="Test321",
    #                      database="test")
    #"外网测试环境":
    msg = SQLServer(server="127.0.0.1",
                         port=0,
                         user="test",
                         password="Test321",
                         database="test")
    #"外网正式环境":
    # self.msg = SQLServer(server="127.0.0.1",
    #                      port=0,
    #                      user="test",
    #                      password="Test321",
    #                      database="test")

    path = os.path.split(os.path.realpath(__file__))[0] + "/SQL"
    # sql文件列表
    files = os.listdir(path)
    fileList = []
    for f in files:
        # 添加文件
        if os.path.splitext(f)[1].lower() == ".sql":
            filename = path + "\\" + f
            msg.GetSqlInfo(filename)
            fileList.append(filename)

if __name__ == '__main__':
    main()
